from fundus.publishers.base_objects import Publisher, PublisherGroup
from fundus.scraping.filter import inverse, lor, regex_filter
from fundus.scraping.url import Sitemap

from .the_portugal_news import ThePortugalNewsParser


class PT(metaclass=PublisherGroup):
    default_language = "pt"

    ThePortugalNews = Publisher(
        name="The Portugal News",
        domain="https://www.theportugalnews.com/",
        parser=ThePortugalNewsParser,
        # There are more languages un the sitemap that could be added in the future
        sources=[
            Sitemap(
                "https://www.theportugalnews.com/sitemap.xml",
                sitemap_filter=lor(regex_filter("category-pages"), inverse(regex_filter("/en/"))),
                languages={"en"},
            ),
        ],
    )



import datetime
import re
from typing import List, Optional

from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, ParserProxy, attribute
from fundus.parser.data import Image
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class BildParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath("//div[@class = 'article-body']/p[position() > 1]")
        _summary_selector = XPath("//div[@class = 'article-body']/p[1]")
        _subheadline_selector = XPath("//div[@data-key = 'article']/h2")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("keywords"))

        @attribute
        def free_access(self) -> bool:
            if (url := self.precomputed.meta.get("og:url")) is not None:
                return re.search(r"/bild-plus/", url) is None
            else:
                return True

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure//img[not(contains(@class, 'teaser') or contains(@class, 'author'))]"),
                caption_selector=XPath("./ancestor::figure//p[@class='fig__caption__text']"),
                author_selector=XPath("./ancestor::figure//div[@class='fig__caption__meta']"),
            )



import datetime
import re
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class SermitsiaqParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath(
            f"//div[contains(@class, 'bodytext')]//p[not(@class='offer-description' or re:test(text(), '^/.*/$'))]",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        )
        _summary_selector = XPath("//h2[@class='subtitle '] ")
        _subheadline_selector = XPath("//div[contains(@class, 'bodytext')]//h3[not(@class='offer-name')]")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"), split_on=["og"])

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def topics(self) -> List[str]:
            return [tag.title() for tag in generic_topic_parsing(self.precomputed.meta.get("article:tag"))]

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                image_selector=XPath("//figure//img[not(@itemprop='image')]"),
                caption_selector=XPath(
                    "./ancestor::*[self::figure or (self::div and contains(@class,'articleHeader'))]"
                    "//figcaption[@itemprop='caption']"
                ),
                author_selector=XPath(
                    "./ancestor::*[self::figure or (self::div and contains(@class,'articleHeader'))]"
                    "//figcaption[@itemprop='author']"
                ),
            )



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class TAParser(ParserProxy):
    class V1(BaseParser):
        _summary_selector = CSSSelector("p.ContentHead_lead____SsS")
        _subheadline_selector = CSSSelector("article > h2")
        _paragraph_selector = CSSSelector(
            "article > p"
            ":not(.ContentHead_lead____SsS)"
            ":not(.Feedback_root__fr_Mi)"
            ":not(.ArticleContainer_agencies__g6Lpj)"
        )

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                caption_selector=XPath("./ancestor::figure//figcaption/div[1]"),
                author_selector=XPath("./ancestor::figure//figcaption/div[2]"),
                lower_boundary_selector=CSSSelector("div.EndOfArticleSectionList_root__LJO7G"),
            )



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class SZParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.datetime(2024, 2, 1).date()
        _paragraph_selector: XPath = CSSSelector("main [itemprop='articleBody'] > p, main .css-korpch > div > ul > li")
        _summary_selector: XPath = CSSSelector("main [data-manual='teaserText']")
        _subheadline_selector: XPath = CSSSelector("main [itemprop='articleBody'] > h3")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
                paragraph_selector=self._paragraph_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.ld.bf_search("headline")

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.ld.bf_search("keywords"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                caption_selector=XPath("./ancestor::figure//figcaption/text()"),
                author_selector=XPath("./ancestor::figure//figcaption/small"),
            )

    class V1_1(V1):
        VALID_UNTIL = datetime.date.today()
        _paragraph_selector = XPath(
            "//div[@itemprop='articleBody'] //p[@data-manual='paragraph' and not(contains(text(), '© dpa-infocom'))]"
        )
        _summary_selector = CSSSelector("main [data-manual='teaserText']")
        _subheadline_selector = XPath(
            "//div[@itemprop='articleBody']//h3[@data-manual='subheadline'] |"
            "//div[@itemprop='articleBody']//h2[@data-manual='subheadline']"
        )



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class LeMondeParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector: CSSSelector = CSSSelector("p[class='article__paragraph ']")
        _summary_selector: XPath = XPath("//p[contains(@class, 'article__desc') or @id='js-summary-live']")
        _subheadline_selector: XPath = XPath("//h2[@class = 'article__sub-title']")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                summary_selector=self._summary_selector,
                subheadline_selector=self._subheadline_selector,
            )

        @attribute
        def title(self) -> Optional[str]:
            # Use the `get` function to retrieve data from the `meta` precomputed attribute
            return self.precomputed.meta.get("og:title")

        @attribute
        def topics(self) -> List[str]:
            return self.precomputed.ld.bf_search("keywords")  # type: ignore

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            # Use the `get` function to retrieve data from the `meta` precomputed attribute
            return generic_date_parsing(self.precomputed.meta.get("og:article:published_time"))

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.meta.get("og:article:author"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=CSSSelector("article"),
                image_selector=XPath("//figure/descendant::img[1]"),
                caption_selector=XPath("./ancestor::figure//figcaption/text()"),
                author_selector=XPath("./ancestor::figure//figcaption/span"),
            )



import datetime
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    generic_topic_parsing,
    image_extraction,
)


class TheTelegraphParser(ParserProxy):
    class V1(BaseParser):
        VALID_UNTIL = datetime.date(2024, 9, 9)
        _paragraph_selector = CSSSelector("div.articleBodyText p")
        _subheadline_selector = CSSSelector("div.articleBodyText h2")
        _summary_selector = CSSSelector("p[itemprop='description']")
        _datetime_selector = CSSSelector("time[itemprop='datePublished']")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            body = extract_article_body_with_selector(
                self.precomputed.doc,
                summary_selector=self._summary_selector,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
            )
            return body

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            datetime_nodes = self._datetime_selector(self.precomputed.doc)
            if datetime_nodes:
                return generic_date_parsing(datetime_nodes[0].get("datetime"))
            return None

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.meta.get("DCSext.author"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def topics(self) -> List[str]:
            return generic_topic_parsing(self.precomputed.meta.get("keywords"))

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                caption_selector=XPath("./ancestor::figure//figcaption/span[1]"),
                relative_urls=True,
            )

    class V1_1(V1):
        VALID_UNTIL = datetime.date.today()

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))



import datetime
import re
from typing import List, Optional

from lxml.cssselect import CSSSelector
from lxml.etree import XPath

from fundus.parser import ArticleBody, BaseParser, Image, ParserProxy, attribute
from fundus.parser.utility import (
    extract_article_body_with_selector,
    generic_author_parsing,
    generic_date_parsing,
    image_extraction,
)


class FrankfurterRundschauParser(ParserProxy):
    class V1(BaseParser):
        _paragraph_selector = XPath(
            "//p[@class='id-StoryElement-paragraph'] | "
            "//p[contains(@class,'id-Article-content-item-paragraph') and text()] |"
            "//div[@class='id-Article-body']//ul/li[not(@class='id-AuthorList-item ')]"
        )
        _summary_selector = CSSSelector(
            "p.id-StoryElement-leadText, p[class='id-Article-content-item id-Article-content-item-summary']"
        )
        _subheadline_selector = CSSSelector("h2.id-StoryElement-crosshead, span.id-Article-content-item-headline-text")

        @attribute
        def body(self) -> Optional[ArticleBody]:
            return extract_article_body_with_selector(
                self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                subheadline_selector=self._subheadline_selector,
                summary_selector=self._summary_selector,
            )

        @attribute
        def authors(self) -> List[str]:
            return generic_author_parsing(self.precomputed.ld.bf_search("author"))

        @attribute
        def publishing_date(self) -> Optional[datetime.datetime]:
            return generic_date_parsing(self.precomputed.ld.bf_search("datePublished"))

        @attribute
        def title(self) -> Optional[str]:
            return self.precomputed.meta.get("og:title")

        @attribute
        def images(self) -> List[Image]:
            return image_extraction(
                doc=self.precomputed.doc,
                paragraph_selector=self._paragraph_selector,
                upper_boundary_selector=CSSSelector("article"),
                author_selector=re.compile(r"©(?P<credits>.+)"),
            )
